@@ -7,11 +7,14 @@ from matplotlib.figure import Figure
 from matplotlib.axes import Axes
 
 import matplotlib as mpl
+from matplotlib.patches import Polygon
 from matplotlib import pyplot as plt
+from matplotlib.colors import Colormap
 import numpy as np
 
 from . import primitives as pr
 from . import constraints as cr
+from . import constructions as cn
 
 ## Functions for plotting geometric primitives
 
@@ -32,8 +35,22 @@ def plot_point(ax: Axes, point: pr.Point, label: Optional[str]=None, **kwargs):
         Additional keyword arguments for plotting
     """
     x, y = point.value
-    ax.plot([x], [y], marker=".", **kwargs)
+    line, = ax.plot([x], [y], marker=".", **kwargs)
+    ax.annotate(label, (x, y), ha='center', **kwargs)
 
+def rotation_from_line(line: pr.Line) -> float:
+    """
+    Return the rotation of a line vector
+    """
+    line_vec = cn.LineVector.assem((line,))
+    unit_vec = line_vec / np.linalg.norm(line_vec)
+
+    # Since `unit_vec` has unit length, the x-component is the cosine
+    theta = 180/np.pi * np.arccos(unit_vec[0])
+    if unit_vec[1] < 0:
+        theta = theta + 180
+
+    return theta
 
 def plot_line(ax: Axes, line: pr.Line, label: Optional[str]=None, **kwargs):
     """
@@ -50,10 +67,16 @@ def plot_line(ax: Axes, line: pr.Line, label: Optional[str]=None, **kwargs):
     **kwargs
         Additional keyword arguments for plotting
     """
-    xs = np.array([point.value[0] for point in line.values()])
-    ys = np.array([point.value[1] for point in line.values()])
-    ax.plot(xs, ys, **kwargs)
+    # Don't plot zero length lines
+    if cn.Length.assem((line,)) != 0:
+        xs = np.array([point.value[0] for point in line.values()])
+        ys = np.array([point.value[1] for point in line.values()])
+        ax.plot(xs, ys, **kwargs)
 
+        xmid = 1/2*xs.sum()
+        ymid = 1/2*ys.sum()
+        theta = rotation_from_line(line)
+        ax.annotate(label, (xmid, ymid), ha='center', va='baseline', rotation=theta, **kwargs)
 
 def plot_polygon(ax: Axes, polygon: pr.Polygon, label: Optional[str]=None, **kwargs):
     """
@@ -70,30 +93,42 @@ def plot_polygon(ax: Axes, polygon: pr.Polygon, label: Optional[str]=None, **kwa
     **kwargs
         Additional keyword arguments for plotting
     """
-    points = [polygon[f"Line0"]["Point0"]] + [
-        polygon[f"Line{ii}"]["Point1"] for ii in range(len(polygon))
+    origin = polygon[f"Line0"]["Point0"].value
+    points = [
+        polygon[f"Line{ii}"]["Point0"] for ii in range(len(polygon))
     ]
-    xs = np.array([point.value[0] for point in points])
-    ys = np.array([point.value[1] for point in points])
+    verts = np.array([point.value for point in points])
+    patch_kwargs = kwargs.copy()
+    patch_kwargs['alpha'] = 0.1*kwargs['alpha']
+    poly_patch = Polygon(verts, closed=True, **patch_kwargs)
+    ax.add_patch(poly_patch)
 
-    (line,) = ax.plot(xs, ys, **kwargs)
+    # (line,) = ax.plot(xs, ys, **kwargs)
     if label is not None:
-        # Plot the label in the lower left corner
+        # Place the label at the first point
         ax.annotate(
             label,
-            (xs[0:1], ys[0:1]),
+            origin,
             xycoords="data",
             xytext=(2.0, 2.0),
             textcoords="offset points",
             ha="left",
             va="bottom",
-            color=line.get_color(),
+            **kwargs
         )
 
-
 def plot_generic_prim(ax: Axes, prim: pr.Primitive, label: Optional[str]=None, **kwargs):
+    pass
+
+def plot_prim(
+    ax: Axes,
+    prim: pr.Primitive,
+    prim_key: str='',
+    max_label_depth: int = 99,
+    **kwargs
+):
     """
-    Plot a generic primitive by recursively plotting any child primitives
+    Recursively plot all child primitives of a generic primitive
 
     Parameters
     ----------
@@ -106,9 +141,48 @@ def plot_generic_prim(ax: Axes, prim: pr.Primitive, label: Optional[str]=None, *
     **kwargs
         Additional keyword arguments for plotting
     """
-    for key, child_prim in prim.items():
-        plot = make_plot(child_prim)
-        plot(ax, child_prim, key)
+    split_key = prim_key.split("/")
+
+    # The primitive heigth is the maximum primitive depth for all child
+    # primitives.
+    prim_height = prim.node_height() + len(split_key) - 1
+
+    # The primitive depth is how far away from the root node the primitive is.
+    depth = len(split_key) - 1
+
+    # Add labels for primitives before a maximum depth
+    if depth > max_label_depth or prim_key == '':
+        label = None
+    else:
+        parent_key = depth*"."
+        label = f"{parent_key}/{split_key[-1]}"
+
+    # Use the prim height to control opacity.
+    # The root primitive has 100% opacity while deeper child primitives
+    # have lower opacity.
+    if prim_height == 0:
+        s = 1
+    else:
+        s = (prim_height - depth)/prim_height
+    alpha = 1*s + 0.2*(1-s)
+
+    plot = make_plot(prim)
+    plot(ax, prim, label=label, alpha=alpha, **kwargs)
+
+    if isinstance(prim, pr.Line):
+        pass
+        # NOTE: Skip plotting point belonging to lines because the labels overlap
+        # TODO: Implement nice plots of points in lines
+        # Should try to avoid label overlap
+    else:
+        for child_key, child_prim in prim.items():
+            plot_prim(
+                ax,
+                child_prim,
+                prim_key=f'{prim_key}/{child_key}',
+                max_label_depth=max_label_depth,
+                **kwargs
+            )
 
 
 ## Functions for plotting arbitrary geometric primitives
@@ -131,7 +205,6 @@ def make_plot(
         This function is one of the above `plot_...` function
         (see `plot_point`, `plot_line`, etc.).
     """
-
     if isinstance(prim, pr.Point):
         return plot_point
     elif isinstance(prim, pr.Line):
@@ -142,7 +215,7 @@ def make_plot(
         return plot_generic_prim
 
 
-def plot_prims(ax: Axes, root_prim: pr.Primitive):
+def plot_prims(ax: Axes, root_prim: pr.Primitive, cmap: Colormap=mpl.colormaps['viridis']):
     """
     Plot all child primitives in a root primitive
 
@@ -153,10 +226,10 @@ def plot_prims(ax: Axes, root_prim: pr.Primitive):
     root_prim: pr.Primitive
         The primitive to plot
     """
-
-    for label, prim in root_prim.items():
-        plot = make_plot(prim)
-        plot(ax, prim, label=label)
+    num_prims = len(root_prim)
+    for ii, (key, prim) in enumerate(root_prim.items()):
+        color = cmap(ii / num_prims)
+        plot_prim(ax, prim, prim_key=key, color=color)
 
 
 def figure_prims(
