@@ -4,7 +4,6 @@ Test geometric onstraints
 
 import pytest
 
-import typing as tp
 from numpy.typing import NDArray
 
 import itertools
@@ -15,128 +14,11 @@ from matplotlib.ticker import MultipleLocator
 
 from mpllayout import primitives as pr
 from mpllayout import constraints as co
+from mpllayout import constructions as con
 from mpllayout import ui
 from mpllayout.containers import Node
 
-
-class GeometryFixtures:
-    """
-    Utilities to help create primitives
-    """
-
-    ## Point creation
-    def make_point(self, coord):
-        """
-        Return a `pr.Point` at the given coordinates
-        """
-        return pr.Point(value=coord)
-
-    def make_relative_point(self, point: pr.Point, displacement: NDArray):
-        """
-        Return a `pr.Point` displaced from a given point
-        """
-        return pr.Point(value=point.value + displacement)
-
-    ## Line creation
-    def make_rotation(self, theta: float):
-        rot_mat = np.array(
-            [[np.cos(theta), -np.sin(theta)], [np.sin(theta), np.cos(theta)]]
-        )
-        return rot_mat
-
-    def make_line(self, origin: NDArray, line_vec: NDArray):
-        """
-        Return a `pr.Line` with given origin and line vector
-        """
-        coords = (origin, origin + line_vec)
-        return pr.Line(value=[], prims=tuple(pr.Point(x) for x in coords))
-
-    def make_relative_line(
-        self, line: pr.Line, translation: NDArray, deformation: NDArray
-    ):
-        """
-        Return a `pr.Line` deformed about it's start point then translated
-        """
-        lineb_vec = line[1].value - line[0].value
-        lineb_vec = deformation @ lineb_vec
-
-        lineb_start = line[0].value + translation
-        return self.make_line(lineb_start, lineb_vec)
-
-    def make_relline_about_mid(
-        self, line: pr.Line, translation: NDArray, deformation: NDArray
-    ):
-        """
-        Return a `pr.Line` deformed about it's midpoint then translated
-        """
-        lineb_vec = line[1].value - line[0].value
-        lineb_vec = deformation @ lineb_vec
-
-        lineb_mid = 1/2*(line[0].value + line[1].value) + translation
-        lineb_start = lineb_mid - lineb_vec/2
-        return self.make_line(lineb_start, lineb_vec)
-
-    ## Quadrilateral creation
-    def make_quad(self, displacement, deformation):
-        """
-        Return a `pr.Quadrilateral` translated and deformed from a unit quadrilateral
-        """
-        # Specify vertices of a unit square, then deform it and translate it
-        verts = np.array([[0, 0], [1, 0], [1, 1], [0, 1]])
-        verts = np.tensordot(verts, deformation, axes=(-1, -1))
-        verts = verts + displacement
-
-        return pr.Quadrilateral(
-            value=[], children=tuple(pr.Point(vert) for vert in verts)
-        )
-
-    def make_quad_grid(
-        self,
-        translation: NDArray,
-        col_margins: NDArray,
-        row_margins: NDArray,
-        col_widths: NDArray,
-        row_heights: NDArray,
-    ):
-        """
-        Return a grid of `Quadrilateral`s with shape (M, N)
-
-        Rows and columns are numbered from top to bottom and left to right, respectively.
-
-        Parameters
-        ----------
-        col_margins, row_margins: NDArray (N-1,), (M-1,)
-            Column and row margins
-        col_widths, row_heights: NDArray (N,), (M,)
-            Column and row dimensions
-        """
-        # Determine translations/transformations needed for each quad
-        col_defs = col_widths[:, None, None] * np.outer([1, 0], [1, 0]) + np.outer(
-            [0, 1], [0, 1]
-        )
-        row_defs = np.outer([1, 0], [1, 0]) + row_heights[:, None, None] * np.outer(
-            [0, 1], [0, 1]
-        )
-
-        cum_col_widths = np.cumsum(
-            np.concatenate((translation[[0]], col_widths[:-1] + col_margins))
-        )
-        col_trans = np.stack([np.array([x, 0]) for x in cum_col_widths], axis=0)
-        cum_row_heights = np.cumsum(
-            np.concatenate((translation[[1]], -row_heights[1:] - row_margins))
-        )
-        row_trans = np.stack([np.array([0, y]) for y in cum_row_heights], axis=0)
-
-        row_args = zip(row_trans, row_defs)
-        col_args = zip(col_trans, col_defs)
-
-        quads = tuple(
-            self.make_quad(drow + dcol, row_def @ col_def)
-            for (drow, row_def), (dcol, col_def) in itertools.product(
-                row_args, col_args
-            )
-        )
-        return quads
+from tests.fixture_primitives import GeometryFixtures
 
 
 class TestPoint(GeometryFixtures):
@@ -159,7 +41,7 @@ class TestPoint(GeometryFixtures):
 
     def test_Fix(self, pointa):
         constraint = co.Fix()
-        res = constraint((pointa,), (pointa.value,))
+        res = constraint((pointa,), pointa.value)
         assert np.all(np.isclose(res, 0))
 
 
@@ -189,34 +71,30 @@ class TestPointPoint(GeometryFixtures):
         self, pointa: pr.Point, pointb: pr.Point, distance: float, direction: NDArray
     ):
         constraint = co.DirectedDistance()
-        res = constraint((pointa, pointb), (distance, direction))
+        res = constraint((pointa, pointb), direction, distance)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
-    def xdistance(self, distance, direction):
-        return distance*direction[0]
-
-    def test_XDistance(
-        self, pointa: pr.Point, pointb: pr.Point, xdistance: float
-    ):
-        res = co.XDistance()((pointa, pointb), (xdistance,))
-        assert np.all(np.isclose(res, 0))
+    def XYDistance(self, axis_name: str):
+        if axis_name == 'x':
+            return co.XDistance
+        else:
+            return co.YDistance
 
     @pytest.fixture()
-    def ydistance(self, distance, direction):
-        return distance*direction[1]
+    def xy_distance(self, distance, direction, axis_dir):
+        return np.dot(distance*direction, axis_dir)
 
-    def test_YDistance(
-        self, pointa: pr.Point, pointb: pr.Point, ydistance: float
+    def test_XYDistance(
+        self, XYDistance, pointa: pr.Point, pointb: pr.Point, xy_distance: float
     ):
-        res = co.YDistance()((pointa, pointb), (ydistance,))
+        res = XYDistance()((pointa, pointb), xy_distance)
         assert np.all(np.isclose(res, 0))
 
-
     def test_Coincident(
         self, pointa: pr.Point,
     ):
-        res = co.Coincident()((pointa, pointa), ())
+        res = co.Coincident()((pointa, pointa))
         assert np.all(np.isclose(res, 0))
 
 
@@ -241,27 +119,22 @@ class TestLine(GeometryFixtures):
         return self.make_line(origin, direction * length)
 
     def test_Length(self, linea, length):
-        res = co.Length()((linea,), (length,))
+        res = co.Length()((linea,), length)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
-    def xlength(self, length, direction):
-        proj_dir = np.array([1, 0])
-        proj_length = length * np.dot(direction, proj_dir)
-        return proj_length
-
-    def test_XLength(self, linea, xlength):
-        res = co.XLength()((linea,), (xlength,))
-        assert np.all(np.isclose(res, 0))
+    def XYLength(self, axis_name:str):
+        if axis_name == 'x':
+            return co.XLength
+        else:
+            return co.YLength
 
     @pytest.fixture()
-    def ylength(self, length, direction):
-        proj_dir = np.array([0, 1])
-        proj_length = length * np.dot(direction, proj_dir)
-        return proj_length
+    def xy_length(self, length: float, direction: NDArray, axis_dir: NDArray):
+        return np.dot(length * direction, axis_dir)
 
-    def test_YLength(self, linea, ylength):
-        res = co.YLength()((linea,), (ylength,))
+    def test_XYLength(self, XYLength, linea, xy_length):
+        res = XYLength()((linea,), xy_length)
         assert np.all(np.isclose(res, 0))
 
 
@@ -293,7 +166,7 @@ class TestLineLine(GeometryFixtures):
         return self.make_relative_line(linea, displacement, self.make_rotation(0))
 
     def test_Parallel(self, linea, line_parallel):
-        res = co.Parallel()((linea, line_parallel), ())
+        res = co.Parallel()((linea, line_parallel))
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
@@ -303,7 +176,7 @@ class TestLineLine(GeometryFixtures):
         )
 
     def test_Orthogonal(self, linea, line_orthogonal):
-        res = co.Orthogonal()((linea, line_orthogonal), ())
+        res = co.Orthogonal()((linea, line_orthogonal))
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture(params=[False, True])
@@ -313,12 +186,12 @@ class TestLineLine(GeometryFixtures):
     @pytest.fixture()
     def line_coincident(self, linea, reverse):
         if reverse:
-            return self.make_line(linea['Point1'].value, -1*co.line_vector(linea))
+            return self.make_line(linea['Point1'].value, -1*con.LineVector.assem((linea,)))
         else:
             return linea
 
     def test_CoincidentLines(self, linea, line_coincident, reverse):
-        res = co.CoincidentLines()((linea, line_coincident), (reverse,))
+        res = co.CoincidentLines()((linea, line_coincident), reverse)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
@@ -328,12 +201,12 @@ class TestLineLine(GeometryFixtures):
 
     @pytest.fixture()
     def relative_length(self, linea, lineb):
-        lengtha = np.linalg.norm(co.line_vector(linea))
-        lengthb = np.linalg.norm(co.line_vector(lineb))
+        lengtha = np.linalg.norm(con.LineVector.assem((linea,)))
+        lengthb = np.linalg.norm(con.LineVector.assem((lineb,)))
         return lengtha/lengthb
 
     def test_RelativeLength(self, linea, lineb, relative_length):
-        res = co.RelativeLength()((linea, lineb), (relative_length,))
+        res = co.RelativeLength()((linea, lineb), relative_length)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
@@ -347,30 +220,33 @@ class TestLineLine(GeometryFixtures):
         return self.make_relative_line(linea, displacement, scale @ rotate)
 
     def test_Angle(self, linea, line_angle, angle):
-        res = co.Angle()((linea, line_angle), (angle,))
+        res = co.Angle()((linea, line_angle), angle)
         assert np.all(np.isclose(res, 0))
 
     def test_Collinear(self, linea):
-        line_vec = co.line_vector(linea)
+        line_vec = con.LineVector.assem((linea,))
 
         lineb = self.make_relative_line(
             linea, np.random.rand()*line_vec, np.diag(np.ones(2))
         )
-        res = co.Collinear()((linea, lineb), ())
+        res = co.Collinear()((linea, lineb))
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
-    def midpoint_distance(self, linea, lineb):
+    def MidpointXYDistance(self, axis_name: str):
+        if axis_name == 'x':
+            return co.MidpointXDistance
+        else:
+            return co.MidpointYDistance
+
+    @pytest.fixture()
+    def midpoint_xy_distance(self, linea, lineb, axis_dir: NDArray):
         midpointa = 1/2*(linea['Point0'].value + linea['Point1'].value)
         midpointb = 1/2*(lineb['Point0'].value + lineb['Point1'].value)
-        return midpointb - midpointa
+        return np.dot(midpointb - midpointa, axis_dir)
 
-    def test_MidpointXDistance(self, linea, lineb, midpoint_distance):
-        res = co.MidpointXDistance()((linea, lineb), (midpoint_distance[0],))
-        assert np.all(np.isclose(res, 0))
-
-    def test_MidpointYDistance(self, linea, lineb, midpoint_distance):
-        res = co.MidpointYDistance()((linea, lineb), (midpoint_distance[1],))
+    def test_MidpointXYDistance(self, MidpointXYDistance, linea, lineb, midpoint_xy_distance):
+        res = MidpointXYDistance()((linea, lineb), midpoint_xy_distance)
         assert np.all(np.isclose(res, 0))
 
 
@@ -400,7 +276,7 @@ class TestLineArray(GeometryFixtures):
 
     @pytest.fixture()
     def lines_collinear(self, linea, num_lines):
-        line_vec = co.line_vector(linea)
+        line_vec = con.LineVector.assem((linea,))
         dists = np.random.rand(num_lines-1)
         return tuple(
             self.make_relative_line(linea, dist*line_vec, np.diag(np.ones(2)))
@@ -408,9 +284,8 @@ class TestLineArray(GeometryFixtures):
         )
 
     def test_CollinearArray(self, linea, lines_collinear):
-        res = co.CollinearArray(1+len(lines_collinear))(
-            (linea,) + lines_collinear, ()
-        )
+        CollinearArray = con.transform_MapType(co.Collinear, (1+len(lines_collinear))*(pr.Line,))
+        res = CollinearArray()((linea,) + lines_collinear)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
@@ -435,8 +310,10 @@ class TestLineArray(GeometryFixtures):
 
     def test_RelativeLengthArray(self, linea, lines_relative, relative_lengths):
 
-        constraint = co.RelativeLengthArray(len(relative_lengths))
-        res = constraint(tuple(lines_relative) + (linea,), (relative_lengths,))
+        constraint = con.transform_map(
+            co.RelativeLength(), (len(lines_relative)+1)*(pr.Line,)
+        )
+        res = constraint(tuple(lines_relative) + (linea,), *relative_lengths)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
@@ -447,25 +324,6 @@ class TestLineArray(GeometryFixtures):
             (linea, lineb) for linea, lineb in zip(lineas, linebs)
         )
 
-    @pytest.fixture()
-    def midpoint_distances(self, line_pairs):
-        def midpoint_distance(linea, lineb):
-            mida = 1/2*(linea['Point0'].value + linea['Point1'].value)
-            midb = 1/2*(lineb['Point0'].value + lineb['Point1'].value)
-            return midb - mida
-        return np.array([midpoint_distance(*line_pair) for line_pair in line_pairs])
-
-    @pytest.fixture()
-    def lines_midpointsarray(self, line_pairs):
-        return tuple(itertools.chain.from_iterable(line_pairs))
-
-    def test_XDistanceMidpointsArray(self, lines_midpointsarray, midpoint_distances):
-        num_pairs = len(lines_midpointsarray) // 2
-        res = co.MidpointXDistanceArray(num_pairs)(
-            lines_midpointsarray, (midpoint_distances[:, 0],)
-        )
-        assert np.all(np.isclose(res, 0))
-
 
 class TestPointLine(GeometryFixtures):
     """
@@ -497,7 +355,7 @@ class TestPointLine(GeometryFixtures):
         origin = line['Point0'].value
         point_vec = point.value - origin
 
-        line_vec = co.line_vector(line)
+        line_vec = con.LineVector.assem((line,))
         line_unit_vec = line_vec / np.linalg.norm(line_vec)
         return np.dot(point_vec, line_unit_vec)
 
@@ -510,7 +368,7 @@ class TestPointLine(GeometryFixtures):
             return self.calc_point_on_line_distance(point, line)
 
     def test_DistanceOnLine(self, point, line, distance_on, reverse):
-        res = co.PointOnLineDistance()((point, line), (distance_on, reverse))
+        res = co.PointOnLineDistance()((point, line), reverse, distance_on)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
@@ -518,7 +376,7 @@ class TestPointLine(GeometryFixtures):
         return distance_on/line_length
 
     def test_RelativeDistanceOnLine(self, point, line, relative_distance_on, reverse):
-        res = co.RelativePointOnLineDistance()((point, line), (relative_distance_on, reverse))
+        res = co.RelativePointOnLineDistance()((point, line), reverse, relative_distance_on)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
@@ -528,7 +386,7 @@ class TestPointLine(GeometryFixtures):
         return np.dot(point.value - line['Point0'].value, orth_unit_vec)
 
     def test_DistanceToLine(self, point, line, distance_to, reverse):
-        res = co.PointToLineDistance()((point, line), (distance_to, reverse))
+        res = co.PointToLineDistance()((point, line), reverse, distance_to)
         assert np.all(np.isclose(res, 0))
 
 
@@ -543,22 +401,38 @@ class TestQuadrilateral(GeometryFixtures):
 
     @pytest.fixture()
     def aspect_ratio(self, quada):
-        width = np.linalg.norm(co.line_vector(quada['Line0']))
-        height = np.linalg.norm(co.line_vector(quada['Line1']))
+        width = np.linalg.norm(con.LineVector.assem((quada['Line0'],)))
+        height = np.linalg.norm(con.LineVector.assem((quada['Line1'],)))
         return width/height
 
     def test_AspectRatio(self, quada: pr.Quadrilateral, aspect_ratio: float):
-        res = co.AspectRatio()((quada,), (aspect_ratio,))
+        res = co.AspectRatio()((quada,), aspect_ratio)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
-    def quad_box(self):
+    def width(self):
+        return np.random.rand()
+
+    @pytest.fixture()
+    def height(self):
+        return np.random.rand()
+
+    @pytest.fixture()
+    def quad_rectangle(self, width, height):
         translation = np.random.rand(2)
-        deformation = np.diag(np.random.rand(2))
+        deformation = np.diag([width, height])
         return self.make_quad(translation, deformation)
 
-    def test_Box(self, quad_box: pr.Quadrilateral):
-        res = co.Box()((quad_box,), ())
+    def test_Box(self, quad_rectangle: pr.Quadrilateral):
+        res = co.Box()((quad_rectangle,))
+        assert np.all(np.isclose(res, 0))
+
+    def test_Width(self, quad_rectangle: pr.Quadrilateral, width: float):
+        res = co.Width()((quad_rectangle,), width)
+        assert np.all(np.isclose(res, 0))
+
+    def test_Height(self, quad_rectangle: pr.Quadrilateral, height: float):
+        res = co.Height()((quad_rectangle,), height)
         assert np.all(np.isclose(res, 0))
 
 
@@ -605,7 +479,7 @@ class TestQuadrilateralQuadrilateral(GeometryFixtures):
         return margin
 
     def test_OuterMargin(self, boxa, boxb, outer_margin, margin_side):
-        res = co.OuterMargin(side=margin_side)((boxa, boxb), (outer_margin,))
+        res = co.OuterMargin(side=margin_side)((boxa, boxb), outer_margin)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
@@ -626,7 +500,7 @@ class TestQuadrilateralQuadrilateral(GeometryFixtures):
         return margin
 
     def test_InnerMargin(self, boxa, boxb, inner_margin, margin_side):
-        res = co.InnerMargin(side=margin_side)((boxa, boxb), (inner_margin,))
+        res = co.InnerMargin(side=margin_side)((boxa, boxb), inner_margin)
         assert np.all(np.isclose(res, 0))
 
 
@@ -663,13 +537,7 @@ class TestQuadrilateralArray(GeometryFixtures):
         rel_row_heights = 3 * np.ones(num_row - 1)
         row_margins = 0.1 * np.ones(num_row - 1)
 
-        grid_kwargs = {
-            "col_widths": rel_col_widths,
-            "row_heights": rel_row_heights,
-            "col_margins": col_margins,
-            "row_margins": row_margins,
-        }
-        return grid_kwargs
+        return (rel_col_widths, rel_row_heights, col_margins, row_margins)
 
     @pytest.fixture()
     def quads_grid(
@@ -680,10 +548,10 @@ class TestQuadrilateralArray(GeometryFixtures):
         origin = np.random.rand(2)
         origin = np.zeros(2)
 
-        col_margins = grid_parameters["col_margins"]
-        row_margins = grid_parameters["row_margins"]
-        rel_col_widths = grid_parameters["col_widths"]
-        rel_row_heights = grid_parameters["row_heights"]
+        rel_col_widths = grid_parameters[0]
+        rel_row_heights = grid_parameters[1]
+        col_margins = grid_parameters[2]
+        row_margins = grid_parameters[3]
 
         origin_width, origin_height = grid_origin_quad_size
         col_widths = origin_width * np.concatenate(([1], rel_col_widths))
@@ -696,7 +564,7 @@ class TestQuadrilateralArray(GeometryFixtures):
     def test_RectilinearGrid(
         self, quads_grid: list[pr.Quadrilateral], grid_shape: tuple[int, int]
     ):
-        res = co.RectilinearGrid(grid_shape)(quads_grid, ())
+        res = co.RectilinearGrid(grid_shape)(quads_grid)
         assert np.all(np.isclose(res, 0))
 
     def test_Grid(
@@ -716,7 +584,7 @@ class TestQuadrilateralArray(GeometryFixtures):
 
         fig.savefig(f"quad_grid_{grid_shape}.png")
 
-        res = co.Grid(grid_shape)(quads_grid, grid_parameters)
+        res = co.Grid(grid_shape)(quads_grid, *grid_parameters)
         assert np.all(np.isclose(res, 0))
 
 
@@ -730,22 +598,24 @@ class TestAxesConstraints(GeometryFixtures):
     def axes_size(self):
         return np.random.rand(2)
 
-    @pytest.fixture(
-        params=[
-            {'bottom': True, 'top': False},
-            {'bottom': False, 'top': True},
-        ]
-    )
-    def xaxis_position(self, request):
+    @pytest.fixture(params=(False, True))
+    def twinx(self, request):
+        return request.param
+
+    @pytest.fixture(params=(False, True))
+    def twiny(self, request):
         return request.param
 
     @pytest.fixture(
-        params=[
-            {'left': True, 'right': False},
-            {'left': False, 'right': True},
-        ]
+        params=['bottom', 'top']
     )
-    def yaxis_position(self, request):
+    def xaxis_side(self, request):
+        return request.param
+
+    @pytest.fixture(
+        params=['left', 'right']
+    )
+    def yaxis_side(self, request):
         return request.param
 
     @pytest.fixture()
@@ -760,8 +630,8 @@ class TestAxesConstraints(GeometryFixtures):
     def axes(
         self,
         axes_size: tuple[float, float],
-        xaxis_position,
-        yaxis_position,
+        xaxis_side,
+        yaxis_side,
         xaxis_height,
         yaxis_width,
         xlabel_position,
@@ -773,26 +643,39 @@ class TestAxesConstraints(GeometryFixtures):
 
         squash_width = np.array([[0, 0], [0, 1]])
 
-        scale = np.diag([axes_width, xaxis_height])
-        if xaxis_position['bottom']:
-            xaxis = self.make_quad(np.array([0, -xaxis_height]), scale)
-        elif xaxis_position['top']:
-            xaxis = self.make_quad(np.array([0, axes_height]), scale)
-        else:
-            raise ValueError()
+        def make_quad_on_hor_side(bottom: bool):
+            scale = np.diag([axes_width, xaxis_height])
+            if bottom:
+                return self.make_quad(np.array([0, -xaxis_height]), scale)
+            else:
+                return self.make_quad(np.array([0, axes_height]), scale)
 
-        scale = np.diag([yaxis_width, axes_height])
-        if yaxis_position['left']:
-            yaxis = self.make_quad(np.array([-yaxis_width, 0]), scale)
-        elif yaxis_position['right']:
-            yaxis = self.make_quad(np.array([axes_width, 0]), scale)
+        if xaxis_side == 'bottom':
+            xbottom = True
         else:
-            raise ValueError()
+            xbottom = False
+
+        xaxis = make_quad_on_hor_side(xbottom)
+        twin_xaxis = make_quad_on_hor_side(not xbottom)
+
+        def make_quad_on_ver_side(left: bool):
+            scale = np.diag([yaxis_width, axes_height])
+            if left:
+                return self.make_quad(np.array([-yaxis_width, 0]), scale)
+            else:
+                return self.make_quad(np.array([axes_width, 0]), scale)
+
+        if yaxis_side == 'left':
+            yleft = True
+        else:
+            yleft = False
+
+        yaxis = make_quad_on_ver_side(yleft)
+        twin_yaxis = make_quad_on_ver_side(not yleft)
 
         def point_from_arclength(line: pr.Line, s: float):
             origin = line['Point0'].value
-            line_vector = co.line_vector(line)
-            return origin + s*line_vector
+            return origin + s*con.LineVector.assem((line,))
 
         xlabel_anchor = self.make_point(
             [point_from_arclength(xaxis['Line0'], xlabel_position)[0], np.random.rand()]
@@ -801,38 +684,50 @@ class TestAxesConstraints(GeometryFixtures):
             [np.random.rand(), point_from_arclength(yaxis['Line1'], ylabel_position)[1]]
         )
 
-        return pr.Axes(
-            prims=(frame, xaxis, xlabel_anchor, yaxis, ylabel_anchor),
-            xaxis=True,
-            yaxis=True
+        twin_xlabel_anchor = self.make_point(
+            [point_from_arclength(xaxis['Line0'], xlabel_position)[0], np.random.rand()]
+        )
+        twin_ylabel_anchor = self.make_point(
+            [np.random.rand(), point_from_arclength(yaxis['Line1'], ylabel_position)[1]]
         )
 
-    def test_PositionXAxis(self, axes, xaxis_position):
-        res = co.PositionXAxis(**xaxis_position)((axes,), ())
+        xaxis_prims = (xaxis, xlabel_anchor)
+        yaxis_prims = (yaxis, ylabel_anchor)
+        twin_xaxis_prims = (twin_xaxis, twin_xlabel_anchor)
+        twin_yaxis_prims = (twin_yaxis, twin_ylabel_anchor)
+
+        prims = (frame, *xaxis_prims, *yaxis_prims, *twin_xaxis_prims, *twin_yaxis_prims)
+
+        return pr.Axes(
+            prims=prims, xaxis=True, yaxis=True, twinx=True, twiny=True
+        )
+
+    def test_PositionXAxis(self, axes, xaxis_side, twinx):
+        res = co.PositionXAxis(side=xaxis_side, twin=twinx)((axes,))
         assert np.all(np.isclose(res, 0))
 
-    def test_PositionYAxis(self, axes, yaxis_position):
-        res = co.PositionYAxis(**yaxis_position)((axes,), ())
+    def test_PositionYAxis(self, axes, yaxis_side):
+        res = co.PositionYAxis(side=yaxis_side)((axes,))
 
         assert np.all(np.isclose(res, 0))
 
-    def test_PositionXAxisLabel(self, axes, xlabel_position):
-        res = co.PositionXAxisLabel()((axes,), (xlabel_position,))
+    def test_PositionXAxisLabel(self, axes, xlabel_position, twinx):
+        res = co.PositionXAxisLabel(twin=twinx)((axes,), xlabel_position)
         assert np.all(np.isclose(res, 0))
 
-    def test_PositionYAxisLabel(self, axes, ylabel_position):
-        res = co.PositionYAxisLabel()((axes,), (ylabel_position,))
+    def test_PositionYAxisLabel(self, axes, ylabel_position, twiny):
+        res = co.PositionYAxisLabel(twin=twiny)((axes,), ylabel_position)
 
         assert np.all(np.isclose(res, 0))
 
-    # NOTE: The `..AxisHeight` constraints have signature [Quadrilateral] but are
+    # NOTE: The `..AxisThickness` constraints have signature [Quadrilateral] but are
     # specialized to axes so I included them here rather than with `TestQuadrilateral`
     @pytest.fixture()
     def axes_mpl(
             self,
             axes_size: tuple[float, float],
-            xaxis_position,
-            yaxis_position,
+            xaxis_side,
+            yaxis_side,
         ):
         width, height = axes_size
         # NOTE: Unit dimensions of the figure are important because `fig.add_axes`
@@ -846,31 +741,31 @@ class TestAxesConstraints(GeometryFixtures):
         ax.set_xlabel("My favourite xlabel")
         ax.set_ylabel("My favourite ylabel")
 
-        if xaxis_position['bottom']:
+        if xaxis_side == 'bottom':
             ax.xaxis.tick_bottom()
-        elif xaxis_position['top']:
+        elif xaxis_side == 'top':
             ax.xaxis.tick_top()
 
-        if yaxis_position['left']:
+        if yaxis_side == 'left':
             ax.yaxis.tick_left()
-        elif yaxis_position['right']:
+        elif yaxis_side == 'right':
             ax.yaxis.tick_right()
 
         return ax
 
     @pytest.fixture()
     def xaxis_height(self, axes_mpl):
-        return co.XAxisHeight.get_xaxis_height(axes_mpl.xaxis)
+        return co.XAxisThickness.get_axis_thickness(axes_mpl.xaxis)
 
-    def test_XAxisHeight(self, axes, axes_mpl):
-        res = co.XAxisHeight()((axes['XAxis'],), (axes_mpl.xaxis,))
+    def test_XAxisThickness(self, axes, axes_mpl):
+        res = co.XAxisThickness()((axes['XAxis'],), axes_mpl.xaxis)
         assert np.all(np.isclose(res, 0))
 
     @pytest.fixture()
     def yaxis_width(self, axes_mpl):
-        return co.YAxisWidth.get_yaxis_width(axes_mpl.yaxis)
+        return co.YAxisThickness.get_axis_thickness(axes_mpl.yaxis)
 
-    def test_YAxisWidth(self, axes, axes_mpl):
-        res = co.YAxisWidth()((axes['YAxis'],), (axes_mpl.yaxis,))
+    def test_YAxisThickness(self, axes, axes_mpl):
+        res = co.YAxisThickness()((axes['YAxis'],), axes_mpl.yaxis)
         assert np.all(np.isclose(res, 0))
 
