@@ -71,12 +71,12 @@ class TestPrimitiveTree:
 
         ## Constrain the axes in a grid
         num_row, num_col = axes_shape
-        grid_param = {
-            "col_margins": (num_col - 1) * [1 / 16],
-            "row_margins": (num_row - 1) * [1 / 16],
-            "col_widths": (num_col - 1) * [1],
-            "row_heights": (num_row - 1) * [1],
-        }
+        grid_param = (
+            (num_col - 1) * [1],
+            (num_row - 1) * [1],
+            (num_col - 1) * [1 / 16],
+            (num_row - 1) * [1 / 16],
+        )
         layout.add_constraint(
             co.Grid(axes_shape),
             tuple(f"Axes{n}" for n in range(num_axes)),
@@ -94,12 +94,12 @@ class TestPrimitiveTree:
         layout.add_constraint(
             co.DirectedDistance(),
             ("Axes0/Line1/Point1", "Figure/Line1/Point1"),
-            (margin_top, np.array([0, 1]))
+            (np.array([0, 1]), margin_top)
         )
         layout.add_constraint(
             co.DirectedDistance(),
             (f"Axes{num_axes-1}/Line1/Point0", "Figure/Line1/Point0"),
-            (margin_bottom, np.array([0, -1]))
+            (np.array([0, -1]), margin_bottom)
         )
 
         # Constrain left/right margins
@@ -108,30 +108,26 @@ class TestPrimitiveTree:
         layout.add_constraint(
             co.DirectedDistance(),
             ("Axes0/Line0/Point0", "Figure/Line0/Point0"),
-            (margin_left, np.array([-1, 0]))
+            (np.array([-1, 0]), margin_left)
         )
         layout.add_constraint(
             co.DirectedDistance(),
             (f"Axes{num_col-1}/Line1/Point1", "Figure/Line1/Point1"),
-            (margin_right, np.array([1, 0]))
+            (np.array([1, 0]), margin_right)
         )
         return layout
 
     def test_assem_constraint_residual(self, layout_grid: lay.Layout):
         layout = layout_grid
 
-        prim_graph, prims = lay.build_prim_graph(layout.root_prim)
-        prim_params = [prim.value for prim in prims]
-
-        prim_tree = layout.root_prim
-        # prim_graph = prim_tree.prim_graph()
+        root_prim = layout.root_prim
         flat_constraints = layout.flat_constraints()
 
         # Plain call
         t0 = time.time()
         for i in range(50):
             solver.assem_constraint_residual(
-                prim_tree, prim_graph, prim_params, *flat_constraints
+                root_prim, *flat_constraints
             )
         t1 = time.time()
         print(f"Duration {t1-t0:.2e} s")
@@ -143,13 +139,13 @@ class TestPrimitiveTree:
         constraints_jit = [jax.jit(constraint) for constraint in constraints]
         flat_constraints_jit = (constraints_jit,) + flat_constraints[1:]
         solver.assem_constraint_residual(
-            prim_tree, prim_graph, prim_params, *flat_constraints_jit
+            root_prim, *flat_constraints_jit
         )
 
         t0 = time.time()
         for i in range(50):
             solver.assem_constraint_residual(
-                prim_tree, prim_graph, prim_params, *flat_constraints_jit
+                root_prim, *flat_constraints_jit
             )
         t1 = time.time()
         print(f"Duration {t1-t0:.2e} s")
@@ -157,20 +153,31 @@ class TestPrimitiveTree:
         # `jax.jit` the overall function
 
         @jax.jit
-        def assem_constraint_residual(prim_params):
+        def assem_constraint_residual(root_prim):
             return solver.assem_constraint_residual(
-                prim_tree, prim_graph, prim_params, *flat_constraints
+                root_prim, *flat_constraints
             )
 
-        assem_constraint_residual(prim_params)
+        assem_constraint_residual(root_prim)
         t0 = time.time()
         for i in range(50):
-            assem_constraint_residual(prim_params)
+            assem_constraint_residual(root_prim)
         t1 = time.time()
         print(f"Duration {t1-t0:.2e} s")
 
-    def test_solve(self, layout: lay.Layout):
-        prim_tree_n, solve_info = solver.solve(layout)
+    @pytest.fixture(
+        params=('newton', 'minimize')
+    )
+    def method(self, request):
+        return request.param
+
+    def test_solve(self, layout: lay.Layout, method: str):
+        t0 = time.time()
+        prim_tree_n, solve_info = solver.solve(
+            layout, method=method, max_iter=100
+        )
+        t1 = time.time()
+        print(f"Solve took {t1-t0:.2e} s")
 
         prim_keys_to_value = {
             key: prim.value for key, prim in cn.iter_flat("", prim_tree_n)
